package com.rjconsultores.ventaboletos.web.gui.controladores.esquemaoperacional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.OperadorEmbarcada;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxPuntoVenta;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderOperadorEmbarcada;
import com.trg.search.Filter;

@Controller("BusquedaOperadorEmbarcadaController")
@Scope("prototype")
public class BusquedaOperadorEmbarcadaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	@Autowired
	private transient PagedListWrapper<OperadorEmbarcada> plwOperador;
	
	private List<PuntoVenta> lspuntoVenta;
	private Paging pagingOperadores;
	private MyListbox operadorList;
	private List<PuntoVenta> lsPuntoVenta;
	private MyComboboxPuntoVenta cmbPuntoVenta;
	private Intbox txtIdOperador;
	private Textbox txtLogin;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		setLsPuntoVenta(new ArrayList<PuntoVenta>());
		
		super.doAfterCompose(comp);
		
		operadorList.setItemRenderer(new RenderOperadorEmbarcada());
		operadorList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				OperadorEmbarcada s = (OperadorEmbarcada) operadorList.getSelected();
				abrirTelaCadastroOperador(s);
			}
		});
		
		refreshLista();
		
		txtIdOperador.focus();		
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void abrirTelaCadastroOperador(OperadorEmbarcada s) {
		if (s == null) {
			return;
		}
		
		Map args = new HashMap();
		args.put("Operador", s);

		openWindow("/gui/esquema_operacional/editarOperadorEmbarcada.zul",
				Labels.getLabel("editarOperadorEmbarcada.titulo"), args, MODAL);
		
	}
	
	@SuppressWarnings("static-access")
	private void refreshLista() {
		HibernateSearchObject<OperadorEmbarcada> sistemaBusqueda =
				new HibernateSearchObject<OperadorEmbarcada>(OperadorEmbarcada.class, pagingOperadores.getPageSize());
		
		sistemaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
				
		if (txtIdOperador.getValue() != null) {
			sistemaBusqueda.addFilterEqual("operadorEmbarcadaId", txtIdOperador.getValue());
		}
		
		String login = txtLogin.getText();
		if (!login.equals("")) {
			sistemaBusqueda.addFilterILike("usuario.claveUsuario", "%" + login.trim().concat("%"));
		}
		
		Comboitem puntoVentaSeleccionada = cmbPuntoVenta.getSelectedItem();

		if (puntoVentaSeleccionada != null) {
			PuntoVenta puntoVenta = (PuntoVenta) puntoVentaSeleccionada.getValue();
			sistemaBusqueda.addFilterSome("usuario.usuarioUbicacionList", Filter.and(Filter.equal("puntoVenta", puntoVenta),
							Filter.equal("activo", Boolean.TRUE)));
		}
		
		plwOperador.init(sistemaBusqueda, operadorList, pagingOperadores);
		
		if (operadorList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaOperadorEmbarcada.titulo"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	public void onClick$btnNovo(Event ev) {
		abrirTelaCadastroOperador(new OperadorEmbarcada());
	}
	
	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public MyListbox getOperadorList() {
		return operadorList;
	}

	public void setOperadorList(MyListbox operadorList) {
		this.operadorList = operadorList;
	}

	public Paging getPagingOperadores() {
		return pagingOperadores;
	}

	public void setPagingOperadores(Paging pagingOperadores) {
		this.pagingOperadores = pagingOperadores;
	}

	public List<PuntoVenta> getLspuntoVenta() {
		return lspuntoVenta;
	}

	public void setLspuntoVenta(List<PuntoVenta> lspuntoVenta) {
		this.lspuntoVenta = lspuntoVenta;
	}

	public List<PuntoVenta> getLsPuntoVenta() {
		return lsPuntoVenta;
	}

	public void setLsPuntoVenta(List<PuntoVenta> lsPuntoVenta) {
		this.lsPuntoVenta = lsPuntoVenta;
	}

	public MyComboboxPuntoVenta getCmbPuntoVenta() {
		return cmbPuntoVenta;
	}

	public void setCmbPuntoVenta(MyComboboxPuntoVenta cmbPuntoVenta) {
		this.cmbPuntoVenta = cmbPuntoVenta;
	}

	public PagedListWrapper<OperadorEmbarcada> getPlwOperador() {
		return plwOperador;
	}

	public void setPlwOperador(PagedListWrapper<OperadorEmbarcada> plwOperador) {
		this.plwOperador = plwOperador;
	}

	public Intbox getTxtIdOperador() {
		return txtIdOperador;
	}

	public void setTxtIdOperador(Intbox txtIdOperador) {
		this.txtIdOperador = txtIdOperador;
	}

	public Textbox getTxtLogin() {
		return txtLogin;
	}

	public void setTxtLogin(Textbox txtLogin) {
		this.txtLogin = txtLogin;
	}
	
}
