package com.rjconsultores.ventaboletos.web.gui.controladores.impressaofiscal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FiscalTotnaofiscalEmpresa;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaTotnaofiscalEmpresaController")
@Scope("prototype")
public class BusquedaTotnaofiscalEmpresaController extends MyGenericForwardComposer  {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	@Autowired
    private transient PagedListWrapper<FiscalTotnaofiscalEmpresa> plwTotnaofiscalEmpresa;
	private Paging pagingTotnaofiscalEmpresa;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	private MyListbox totnaofiscalEmpresaList;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		totnaofiscalEmpresaList.setItemRenderer(new TotnaofiscalEmpresaListItemRenderer());
		totnaofiscalEmpresaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FiscalTotnaofiscalEmpresa totnaofiscalEmpresa = (FiscalTotnaofiscalEmpresa) totnaofiscalEmpresaList.getSelected();
				visualizaTotnaofiscalEmpresa(totnaofiscalEmpresa);
			}
		});
		
		actualizaTotnaofiscalEmpresaList();
	}
	
	private void visualizaTotnaofiscalEmpresa(FiscalTotnaofiscalEmpresa totnaofiscalEmpresa){
		
		if (totnaofiscalEmpresa == null)
			return;
		
		Map args = new HashMap();
		args.put("totnaofiscalEmpresa", totnaofiscalEmpresa);
		args.put("totnaofiscalEmpresaList", totnaofiscalEmpresaList);

		openWindow("/gui/impressaofiscal/editarTotnaofiscalEmpresa.zul",
				Labels.getLabel("editarTotnaofiscalEmpresaController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaTotnaofiscalEmpresaList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaTotnaofiscalEmpresaList();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaTotnaofiscalEmpresa(new FiscalTotnaofiscalEmpresa());
	}
	
	private void actualizaTotnaofiscalEmpresaList() {
		HibernateSearchObject<FiscalTotnaofiscalEmpresa> editarTotnaofiscalEmpresaBusqueda =
				new HibernateSearchObject<FiscalTotnaofiscalEmpresa>(FiscalTotnaofiscalEmpresa.class, pagingTotnaofiscalEmpresa.getPageSize());

		editarTotnaofiscalEmpresaBusqueda.addFilterEqual(ACTIVO, true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			editarTotnaofiscalEmpresaBusqueda.addFilterEqual("empresa", empresa);
		} else {
			editarTotnaofiscalEmpresaBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		plwTotnaofiscalEmpresa.init(editarTotnaofiscalEmpresaBusqueda, totnaofiscalEmpresaList, pagingTotnaofiscalEmpresa);

		if (totnaofiscalEmpresaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAidfController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}
}
