package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioBPeSubstituido;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderEstadoSimple;
import com.trg.search.Filter;

@Controller("relatorioBPeSubstituidoController")
@Scope("prototype")
public class RelatorioBPeSubstituidoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	private Datebox dtInicio;
	private Datebox dtFim; 
	private MyComboboxEstandar cmbEmpresa;
	private MyListbox estadoList;

	private List<Empresa> lsEmpresa;
	private List<Estado> lsEstado;

	@Autowired
	private DataSource dataSourceRead;
	@Autowired
	private EmpresaService empresaService;
	@Autowired
	private EstadoService estadoService;
	
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private ArrayList<PuntoVenta> lsNumPuntoVenta = new ArrayList<PuntoVenta>();
	private MyListbox puntoVentaList;
	private Paging pagingPuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	private Textbox txtPalavraPesquisa;
	private MyListbox puntoVentaSelList;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		lsEstado = estadoService.obtenerTodos();
		super.doAfterCompose(comp);
		
		estadoList.setItemRenderer(new RenderEstadoSimple());
		estadoList.setData(lsEstado);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio() throws Exception {
		Map<String, Object> parametros = new HashMap<String, Object>();
		StringBuilder filtro = new StringBuilder();

		if (!validar()) {
			return;
		}

		if (dtInicio.getValue() != null && dtFim.getValue() != null) {
			filtro.append("Data: ")
					.append(DateUtil.getStringDate(dtInicio.getValue(), "dd/MM/yyyy"))
					.append(" - ")
					.append(DateUtil.getStringDate(dtFim.getValue(), "dd/MM/yyyy"))
					.append(";");
			parametros.put("DATA_INICIO", (java.util.Date) dtInicio.getValue());
			parametros.put("DATA_FIM", (java.util.Date) dtFim.getValue());
		}

		parametros.put("nome_relatorio", Labels.getLabel("relatorioBPeSubstituidoController.window.title"));
		parametros.put("usuario", UsuarioLogado.getUsuarioLogado().getUsuarioId().toString());
		parametros.put("usuario_nome", UsuarioLogado.getUsuarioLogado().getNombusuario());

		filtro.append("UF: ");
		String estadosIds = "";
		String UFs = "";
		List<Object> lsEstadosSelecionados = estadoList.getItensSelecionados();

		if (!lsEstadosSelecionados.isEmpty()) {
			for (int i = 0; i < lsEstadosSelecionados.size(); i++) {
				Estado estado = (Estado) lsEstadosSelecionados.get(i);
				UFs = UFs + estado.getCveestado() + ",";
				estadosIds = estadosIds + estado.getCodibge() + ",";
			}

			estadosIds = estadosIds.substring(0, estadosIds.length() - 1);
			UFs = UFs.substring(0, UFs.length() - 1);
			parametros.put("ESTADOS_ID", estadosIds);
			filtro.append(UFs).append(";");
		} else {
			filtro.append("Todos; ");
		}		
		
		filtro.append("Agência: ");
		String pvIds = "";
		String PVs = "";
		lsNumPuntoVenta = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));

		if (lsNumPuntoVenta.size() > 0) {
			for (PuntoVenta p : lsNumPuntoVenta) {
				PVs = PVs + p.getNombpuntoventa() + ",";
				pvIds = pvIds + p.getPuntoventaId() + ",";
			}
			
			pvIds = pvIds.substring(0, pvIds.length() - 1);
			PVs = PVs.substring(0, PVs.length() - 1);
			parametros.put("PUNTOVENTA_ID", pvIds);
			filtro.append(PVs).append(";");
		} else {
			filtro.append("Todas; ");
		}	
		
		
		filtro.append("Empresa: ");	
		if (cmbEmpresa.getSelectedItem() == null ) {				
			filtro.append(" Todas; ");
		}else{
			Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
			if (itemEmpresa != null) {
				Empresa empresa = (Empresa) itemEmpresa.getValue();
				parametros.put("EMPRESA_ID", String.valueOf(empresa.getEmpresaId()));
				filtro.append(empresa.getNombempresa()).append(";");
			} else {
				filtro.append(" Todas; ");
			}
		}
		
		parametros.put("FILTROS", filtro.toString());
		
		Relatorio relatorio = new RelatorioBPeSubstituido(parametros, dataSourceRead.getConnection());
		
		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("relatorioBPeSubstituidoController.window.title"), args, MODAL);
	}

	private boolean validar() {
		
		try {
			if (dtInicio.getValue() == null || dtFim.getValue() == null) {
				Messagebox.show(Labels.getLabel("relatorioBPeSubstituidoController.data.obrigatoria"),
						Labels.getLabel("relatorioBPeSubstituidoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
				return false;
			}
			
			if(dtInicio.getValue().after(dtFim.getValue())){
				Messagebox.show(Labels.getLabel("relatorioBPeSubstituidoController.MSG.dataInicialMaiorFinal"),
						Labels.getLabel("relatorioBPeSubstituidoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
				return false;
			}			
			
		} catch (InterruptedException ex) {
			return false;
		}
		return true;
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}
	
	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVentaSel);
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVentaSel = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVentaSel);
	}

	public void onSelect$puntoVentaList(Event ev) {
	}

	public void onClick$btnLimpar(Event ev) {
		limparPesquisaAgencia();
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}
	
	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class,
						pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterOr(Filter.like("nombpuntoventa", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")), Filter.like("numPuntoVenta", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")));

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioReceitaDiariaAgenciaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	private void limparPesquisaAgencia() {
		puntoVentaList.clearSelection();

		lsNumPuntoVenta.clear();
		this.bbPesquisaPuntoVenta.setValue("");
	}

	public Datebox getDtInicio() {
		return dtInicio;
	}

	public void setDtInicio(Datebox dtInicio) {
		this.dtInicio = dtInicio;
	}

	public Datebox getDtFim() {
		return dtFim;
	}

	public void setDtFim(Datebox dtFim) {
		this.dtFim = dtFim;
	}

	public MyComboboxEstandar getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEstandar cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyListbox getEstadoList() {
		return estadoList;
	}

	public void setEstadoList(MyListbox estadoList) {
		this.estadoList = estadoList;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

	public DataSource getDataSourceRead() {
		return dataSourceRead;
	}

	public void setDataSourceRead(DataSource dataSourceRead) {
		this.dataSourceRead = dataSourceRead;
	}

	public EstadoService getEstadoService() {
		return estadoService;
	}

	public void setEstadoService(EstadoService estadoService) {
		this.estadoService = estadoService;
	}
	
}
