package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioVendasParcelamento;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiro;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiroSelecionados;

@Controller("relatorioVendasParcelamentoController")
@Scope("prototype")
public class RelatorioVendasParcelamentoController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;
	private static Logger log = LogManager.getLogger(RelatorioVendasParcelamentoController.class);

	@Autowired
	private DataSource dataSourceRead;
	@Autowired 
	private EmpresaService empresaService;
	
	private Datebox datInicial;
	private Datebox datFinal;
	private Intbox parInicial;
	private Intbox parFinal;
	private MyComboboxEstandar cmbEmpresa;
	private Bandbox bbPesquisaPuntoVenta;
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private MyTextbox txtNombrePuntoVenta;
	private MyListbox puntoVentaList;
	private MyListbox puntoVentaSelList;
	private Paging pagingPuntoVenta;
	
	private List<Empresa> lsEmpresas;
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void executarRelatorio() throws Exception {
		try {
			Map<String, Object> parametros = new HashMap<String, Object>();
			
			Empresa empresa =  cmbEmpresa.getSelectedItem() != null ? (Empresa)cmbEmpresa.getSelectedItem().getValue() : null;
			
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			parametros.put("fecInicio", sdf.format(this.datInicial.getValue()));
			parametros.put("fecFinal", sdf.format(this.datFinal.getValue()));
			parametros.put("parInicial", this.parInicial.getValue());
			parametros.put("parFinal", this.parFinal.getValue());
			if (empresa != null){
				parametros.put("empresa", empresa.getEmpresaId());
				parametros.put("empresaNome", empresa.getNombempresa());
			}
			parametros.put("TITULO", Labels.getLabel("indexController.mniRelatorioVendasParcelamento.label"));
			
			String puntoVentaIds = "";
			String puntoVentas = "";
			List<PuntoVenta> lsPuntoVentaSelecionados = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));
			if (lsPuntoVentaSelecionados.isEmpty()) {
				puntoVentas = "Todas";
			} else {
				for (int i = 0; i < lsPuntoVentaSelecionados.size(); i++) {
					PuntoVenta puntoVenta = lsPuntoVentaSelecionados.get(i);
					puntoVentas = puntoVentas + puntoVenta.getNombpuntoventa() + ",";
	
					puntoVentaIds = puntoVentaIds + puntoVenta.getPuntoventaId() + ",";
				}
	
				// removendo ultima virgula
				puntoVentaIds = puntoVentaIds.substring(0, puntoVentaIds.length() - 1);
				puntoVentas = puntoVentas.substring(0, puntoVentas.length() - 1);
				parametros.put("NUMPUNTOVENTA", puntoVentaIds);
			}
			
			Relatorio relatorio = new RelatorioVendasParcelamento(parametros, dataSourceRead.getConnection());
	
			Map<String, Object> args = new HashMap<String, Object>();
			args.put("relatorio", relatorio);
	
			openWindow("/component/reportView.zul",
					Labels.getLabel("indexController.mniRelatorioVendasParcelamento.label"), args, MODAL);
		}catch (Exception e) {
			log.error(e);
			throw e;
		}

	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresas = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		puntoVentaList.setItemRenderer(new RenderRelatorioVendasBilheteiro());
		puntoVentaSelList.setItemRenderer(new RenderRelatorioVendasBilheteiroSelecionados());
	}
	
	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class, pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterILike("nombpuntoventa", "%" + txtNombrePuntoVenta.getValue() + "%");
		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioVendasBilheteiroController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVenta);
	}

	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVenta);
	}

	public void onClick$btnLimpar(Event ev) {
		puntoVentaList.setData(new ArrayList<PuntoVenta>());

		bbPesquisaPuntoVenta.setText("");
	}


	public MyComboboxEstandar getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEstandar cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> lsEmpresas) {
		this.lsEmpresas = lsEmpresas;
	}
	
	

}
